@@ -3,25 +3,19 @@ import math
 import numpy as np
 from scipy.optimize import root
 from scipy import linalg as lin
-from numpy.polynomial import polynomial as P
 
-from AlgebraicNumber.utils import simplify, mahler_separation_bound
-
-# from AlgebraicNumber.inria_utils import inria_mul as an_mul
-# from AlgebraicNumber.inria_utils import inria_add as an_add
-from AlgebraicNumber.utils import kron_mul as an_mul
-from AlgebraicNumber.utils import kron_add as an_add
+from AlgebraicNumber.QPolynomial import QPolynomial
+from AlgebraicNumber.utils import *
+from AlgebraicNumber.inria_utils import inria_mul as an_mul
+from AlgebraicNumber.inria_utils import inria_add as an_add
 
 
 class AlgebraicNumber(object):
     """
-    
+
     Examples:
       >>> a = AlgebraicNumber.imaginary()
-      >>> print(a)
-                              2
-      AlgebraicNumber(1j), 1 X + 1
-      
+
     """
 
     @classmethod
@@ -41,80 +35,108 @@ class AlgebraicNumber(object):
         return AlgebraicNumber([1, 0, 1], 1j)
 
     def __init__(self, coeff, approx, _nosimp=False):
-        self.coeff = coeff
-        self.poly = P.Polynomial(coeff)
+        if isinstance(coeff, QPolynomial):
+            self.poly = coeff
+        else:
+            self.poly = QPolynomial(coeff=coeff)
+
         self.approx = self.eval(approx)
         if not _nosimp:
             self._simplify()
 
     def _simplify(self):
-        c = simplify(self.coeff, self.approx)
+        self.poly = simplify(self.poly, self.approx)
 
-        self.coeff = c
-        self.poly = P.Polynomial(c)
+    @property
+    def coeff(self):
+        p = self.poly.getCoefficients()
+        return p
 
     def eval(self, approx=None):
         if approx is None:
             approx = self.approx
 
-        def fun(X):
-            x, y = X
-            z = x + 1j * y
-            P = self.poly(z)
-            return [np.real(P), np.imag(P)]
+        r = self.poly.roots()
+        k = np.argmin(np.abs(r - approx))
+
+        return r[k]
 
-        sol = root(fun, x0=[np.real(approx), np.imag(approx)])
-        if sol.success:
-            x, y = sol.x
-            z = x + 1j * y
-            return z
-        else:
-            print(sol)
-            raise ValueError
-    
     def plotRoots(self, axe=None, **kwargs):
-        '''Plots the roots of the minimal polynomial of the number
-        
+        """Plots the roots of the minimal polynomial of the number
+
         Args:
           axe
             If given, a matplotlib axe to draw on. By default, plotRoots creates it
           kwargs
             List of arguments to format the plot. Must not specify linestyle.
-            
-        '''
+
+        """
         if axe is None:
             import matplotlib.pyplot as plt
+
             fig = plt.figure()
             axe = fig.add_subplot(111)
             axe.grid(True)
             show = True
         else:
             show = False
-        
-        if not 'marker' in kwargs.keys():
-            kwargs['marker'] = 'o'
-            
-        rc = P.polyroots(self.coeff)
-        axe.plot(np.real(rc), np.imag(rc), linestyle='', **kwargs)
-        axe.plot([np.real(self.approx)], [np.imag(self.approx)], linestyle='', marker='*')
-        
+
+        if not "marker" in kwargs.keys():
+            kwargs["marker"] = "o"
+
+        rc = self.poly.roots()
+        axe.plot(np.real(rc), np.imag(rc), linestyle="", **kwargs)
+        axe.plot(
+            [np.real(self.approx)], [np.imag(self.approx)], linestyle="", marker="*"
+        )
+
         if show:
             plt.show()
 
-    def inverse(self):
+    def pow(self, p: int, q: int = 1) -> "AlgebraicNumber":
+        r"""If :math:`\alpha` is an alebraic number, computes
+
+        .. math::
+            \alpha^{p/q}
+
+        Examples:
+          >>> a = AlgebraicNumber([-2, 0, 1], 1.414)
+          >>> a.pow(2).poly.printCoeff()
+          '[-2,1]'
+          >>> a.pow(1,2).poly.printCoeff()
+          '[-2,0,0,0,1]'
+
+        """
+        if q == 0:
+            raise ZeroDivisionError
+
+        res = AlgebraicNumber.unity()
+        for i in range(p):
+            res = res * self
+
+        h = res.poly
+        Xq = QPolynomial(coeff=[0] * q + [1])
+
+        res = h.compose(Xq)
+
+        res = AlgebraicNumber(res, self.approx ** (p / q))
+
+        return res
+
+    def inverse(self) -> "AlgebraicNumber":
         ZERO = AlgebraicNumber.zero()
         if self == ZERO:
             raise ZeroDivisionError
-            
-        coeff = self.coeff
-        
-        res = AlgebraicNumber(coeff[-1::-1], 1 / self.approx)
+
+        p = self.poly.getCoefficients()
+
+        res = AlgebraicNumber(p[-1::-1], 1 / self.approx)
 
         return res
 
     def __eq__(self, b):
-        sep_a = mahler_separation_bound(self.coeff)
-        sep_b = mahler_separation_bound(b.coeff)
+        sep_a = self.poly.mahler_separation_bound()
+        sep_b = b.poly.mahler_separation_bound()
         eps = min(sep_a, sep_b) / 2
         eq = np.abs(self.approx - b.approx) < eps
 
@@ -124,8 +146,7 @@ class AlgebraicNumber(object):
         return not self.__eq__(b)
 
     def __repr__(self):
-        p = np.poly1d(self.coeff[-1::-1], variable="X")
-        s = str(p)
+        s = str(self.poly)
         elem = s.split("\n")
         info = "%s(%s), " % (self.__class__.__name__, self.approx)
         n = len(info)
@@ -138,15 +159,15 @@ class AlgebraicNumber(object):
 
     def __mul__(self, b):
         """
-        
+
         >>> sqrt_2 = AlgebraicNumber([-4,0,2], 1.4)
         >>> sqrt_3 = AlgebraicNumber([-9,0,3], 1.7)
         >>> p = sqrt_2*sqrt_3
-        >>> p.coeff
-        array([-6,  0,  1]...
-        
+        >>> p.poly.printCoeff()
+        '[-6,0,1]'
+
         """
-        Q = an_mul(self.coeff, b.coeff)
+        Q = an_mul(self.poly, b.poly)
 
         res = AlgebraicNumber(Q, self.approx * b.approx)
 
@@ -154,32 +175,30 @@ class AlgebraicNumber(object):
 
     def __truediv__(self, b):
         """
-        
+
         >>> sqrt_2 = AlgebraicNumber([-4,0,2], 1.4)
         >>> sqrt_3 = AlgebraicNumber([-9,0,3], 1.7)
         >>> p = sqrt_2/sqrt_3
-        >>> p.coeff
-        array([-2,  0,  3]...
-        
+        >>> p.poly.printCoeff()
+        '[-2,0,3]'
+
         """
         ib = b.inverse()
         return self * ib
 
     def __neg__(self):
         """
-        
+
         >>> sqrt_2 = AlgebraicNumber([-4,0,2], 1.4)
         >>> p = -sqrt_2
-        >>> p.coeff
-        array([-2,  0,  1]...
-        
+        >>> p.poly.printCoeff()
+        '[-2,0,1]'
+
         """
-        n = len(self.coeff)
-        coeff = np.array(self.coeff)
+        mx = QPolynomial(coeff=[0, -1])
+        p2 = self.poly.compose(mx)
 
-        R = coeff * (-1) ** np.arange(n)
-
-        res = AlgebraicNumber(list(R), -self.approx)
+        res = AlgebraicNumber(p2, -self.approx)
 
         return res
 
@@ -189,18 +208,18 @@ class AlgebraicNumber(object):
 
     def __add__(self, b):
         """
-        
+
         >>> sqrt_2 = AlgebraicNumber([-4,0,2], 1.4)
         >>> sqrt_3 = AlgebraicNumber([-9,0,3], 1.7)
         >>> p = sqrt_2+sqrt_3
-        >>> p.coeff
-        array([  1,   0, -10,   0,   1]...
+        >>> p.poly.printCoeff()
+        '[1,0,-10,0,1]'
         >>> ref = np.sqrt(2) + np.sqrt(3)
         >>> np.abs(p.approx - ref) < 1e-10
         True
-        
+
         """
-        Q = an_add(self.coeff, b.coeff)
+        Q = an_add(self.poly, b.poly)
 
         res = AlgebraicNumber(Q, self.approx + b.approx)
 
@@ -208,18 +227,16 @@ class AlgebraicNumber(object):
 
     def conj(self):
         """
-        
-        >>> z = AlgebraicNumber.unity() + AlgebraicNumber.imaginary()
-        >>> z.coeff
-        array([ 2, -2,  1]...
-        >>> p = z*z.conj()
-        >>> p.coeff
-        array([-2,  1]...
-        
-        """
-        coeff = self.coeff
 
-        res = AlgebraicNumber(coeff, np.conj(self.approx))
+        >>> z = AlgebraicNumber.unity() + AlgebraicNumber.imaginary()
+        >>> z.poly.printCoeff()
+        '[2,-2,1]'
+        >>> p = z*z.conj()
+        >>> p.poly.printCoeff()
+        '[-2,1]'
+
+        """
+        res = AlgebraicNumber(self.poly, np.conj(self.approx))
 
         return res
 
